@@ -6,7 +6,7 @@
 #' posterior probability of being assigned to that class.
 #' @param x Object to plot.
 #' @param variables Which variables to plot. If NULL, plots all variables that
-#' are present in all Mplus models.
+#' are present in all models.
 #' @param bw Logical. Whether to make a black and white plot (for print) or a
 #' color plot. Defaults to FALSE, because these density plots are hard to read
 #' in black and white.
@@ -57,6 +57,7 @@ plot_density <-
     }
 
 
+#' @method plot_density default
 #' @export
 plot_density.default <-
     function(x,
@@ -68,37 +69,10 @@ plot_density.default <-
 
         plot_df <- x
         # Plot figure
-        if (bw) {
-            if (conditional) {
-                plot_df <- plot_df[-which(plot_df$Class == "Total"),]
-                density_plot <-
-                    ggplot(plot_df,
-                           aes_string(x = "Value", y = "..count..", fill = "Class", weight = "Probability")) +
-                    geom_density(position = "fill") + scale_fill_grey(start = 0.2, end = 0.8)
-            } else{
-                density_plot <-
-                    ggplot(plot_df,
-                           aes_string(x = "Value", linetype = "Class", weight = "Probability")) +
-                    geom_density()
-            }
-        } else{
-            if (conditional) {
-                plot_df <- plot_df[-which(plot_df$Class == "Total"),]
-                density_plot <-
-                    ggplot(plot_df,
-                           aes_string(x = "Value", y = "..count..", fill = "Class", weight = "Probability")) +
-                    geom_density(position = "fill")
-            } else{
-                density_plot <-
-                    ggplot(plot_df,
-                           aes_string(x = "Value",
-                                      fill = "Class",
-                                      colour = "Class",
-                                      weight = "Probability"
-                           )) +
-                    geom_density(alpha = alpha)
-            }
-        }
+        Args <- as.list(match.call()[-1])
+        Args <- Args[which(names(Args) %in% c("variables", "bw", "conditional", "alpha"))]
+        Args <- c(Args, list(plot_df = plot_df))
+        density_plot <- do.call(.plot_density_fun, Args)
         # Relabel facets
         label_facets <- c(levels(plot_df$Variable), levels(plot_df$Title))
         names(label_facets) <- label_facets
@@ -125,15 +99,14 @@ plot_density.default <-
         }
 
         density_plot <- density_plot +
-            theme_bw() +
-            scale_x_continuous(expand = c(0, 0)) +
-            scale_y_continuous(expand = c(0, 0))
+            theme_bw()
 
         suppressWarnings(print(density_plot))
         return(invisible(density_plot))
     }
 
 
+#' @method plot_density tidyLPA
 #' @export
 plot_density.tidyLPA <-
     function(x,
@@ -141,18 +114,15 @@ plot_density.tidyLPA <-
              bw = FALSE,
              conditional = FALSE,
              alpha = .2,
-             facet_labels = NULL) {
-        Args <- as.list(match.call()[-1])
-        x[sapply(x, function(i){is.null(i[["dff"]])})] <- NULL
-        # Check if all variables (except CPROBs) are identical across models
-        plot_df <- lapply(x, function(x)
-            as.data.frame(x$dff))
+             facet_labels = NULL,
+             ...) {
 
-        var_names <-
-            names(plot_df[[1]])[-grep("^(model_number|classes_number|CPROB\\d+|Class)$",
-                                      names(plot_df[[1]]))]
+        Args <- as.list(match.call()[-c(1,2)])
 
         # If no variables have been specified, use all variables
+        var_names <-
+            names(x[[1]]$dff)[-grep("^(model_number|classes_number|CPROB\\d+|Class)$",
+                                      names(x[[1]]$dff))]
         if (is.null(variables)) {
             variables <- var_names
         } else {
@@ -161,45 +131,88 @@ plot_density.tidyLPA <-
         if (!length(variables))
             stop("No valid variables provided.")
 
-        plot_df <-
-            lapply(plot_df, function(x) {
-                x[, which(names(x) %in% c(grep("^CPROB", names(x), value = TRUE), variables))]
-            })
-        plot_df <- lapply(plot_df, function(x) {
-            if (length(grep("^CPROB", names(x))) == 1) {
-                names(x) <- gsub("^CPROB1", "Probability.Total", names(x))
-                x
-            } else {
-                names(x) <- gsub("^CPROB", "Probability.", names(x))
-                data.frame(x, Probability.Total = 1)
-            }
+        Args[["variables"]] <- variables
+        Args[["x"]] <- force(x)
+        Args[["x"]] <- do.call(.extract_density_data, Args[c("x", "variables")])
+        do.call("plot_density", Args)
+    }
 
-        })
+#' @method plot_density tidyProfile
+#' @export
+plot_density.tidyProfile <-
+    function(x,
+             variables = NULL,
+             bw = FALSE,
+             conditional = FALSE,
+             alpha = .2,
+             facet_labels = NULL,
+             ...) {
+        Args <- as.list(match.call()[-c(1,2)])
 
-        for (i in names(plot_df)) {
-            plot_df[[i]][, grep("^Probability", names(plot_df[[i]]))] <-
-                lapply(plot_df[[i]][grep("^Probability", names(plot_df[[i]]))], function(x) {
-                    x / length(x)
-                })
+        # If no variables have been specified, use all variables
+        var_names <-
+            names(x$dff)[-grep("^(model_number|classes_number|CPROB\\d+|Class)$",
+                                    names(x$dff))]
+        if (is.null(variables)) {
+            variables <- var_names
+        } else {
+            variables <- variables[which((variables) %in% (var_names))]
         }
+        if (!length(variables))
+            stop("No valid variables provided.")
+
+        Args[["variables"]] <- variables
+        Args[["x"]] <- force(x)
+        Args[["x"]] <- do.call(.extract_density_data, Args[c("x", "variables")])
+        do.call("plot_density", Args)
+    }
+
+.extract_density_data <- function(x,
+                                  variables = NULL, longform = TRUE){
+    if(inherits(x, "tidyProfile")){
+        x <- list(x)
+    }
+    x[sapply(x, function(i){is.null(i[["dff"]])})] <- NULL
+    # Check if all variables (except CPROBs) are identical across models
+    plot_df <- lapply(x, function(x)
+        as.data.frame(x$dff))
 
 
-        plot_df <- lapply(plot_df, function(x) {
-            reshape(
-                x,
-                direction = "long",
-                varying =
-                    grep("^Probability", names(x), value = TRUE),
-                timevar = "Class",
-                idvar = "ID"
-            )
+    plot_df <-
+        lapply(plot_df, function(x) {
+            x <- x[, which(names(x) %in% c(grep("^CPROB", names(x), value = TRUE), variables))]
+            names(x) <- gsub("^CPROB", "Probability.", names(x))
+            data.frame(x, Probability.Total = 1)
         })
 
+    for (i in names(plot_df)) {
+        plot_df[[i]][, grep("^Probability", names(plot_df[[i]]))] <-
+            lapply(plot_df[[i]][grep("^Probability", names(plot_df[[i]]))], function(x) {
+                x / length(x)
+            })
+    }
+
+    plot_df <- lapply(plot_df, function(x) {
+        reshape(
+            x,
+            direction = "long",
+            varying =
+                grep("^Probability", names(x), value = TRUE),
+            timevar = "Class",
+            idvar = "ID"
+        )
+    })
+
+    if(length(plot_df) > 1){
         plot_df <-
             do.call(rbind, lapply(names(plot_df), function(x) {
                 data.frame(Title = gsub("_", " ", x), plot_df[[x]])
             }))
+    } else {
+        plot_df <- data.frame(Title = "", plot_df[[1]])
+    }
 
+    if(longform){
         variable_names <-
             which(!(
                 names(plot_df) %in% c("Title", "Class", "Probability", "ID")
@@ -224,10 +237,60 @@ plot_density.tidyLPA <-
         )[, c("Title", "Variable", "Value", "Class", "Probability")]
 
         plot_df$Variable <- factor(plot_df$Variable)
-        plot_df$Class <- factor(plot_df$Class)
-        plot_df$Class <-
-            ordered(plot_df$Class, levels = c("Total", levels(plot_df$Class)[-length(levels(plot_df$Class))]))
-        Args[["variables"]] <- variables
-        Args[["x"]] <- plot_df
-        do.call("plot_density", Args)
     }
+
+    plot_df$Class <- factor(plot_df$Class)
+    plot_df$Class <-
+        ordered(plot_df$Class, levels = c("Total", levels(plot_df$Class)[-length(levels(plot_df$Class))]))
+    plot_df
+}
+
+.plot_density_fun <- function(plot_df, variables, bw = FALSE, conditional = FALSE, alpha = .2){
+    if (bw) {
+        if (conditional) {
+            plot_df <- plot_df[-which(plot_df$Class == "Total"),]
+            density_plot <-
+                ggplot(plot_df,
+                       aes_string(x = "Value", y = "..count..", fill = "Class", weight = "Probability")) +
+                geom_density(position = "fill") + scale_fill_grey(start = 0.2, end = 0.8)
+        } else{
+            density_plot <-
+                ggplot(plot_df,
+                       aes_string(x = "Value", linetype = "Class", weight = "Probability")) +
+                geom_density()
+        }
+    } else{
+        if (conditional) {
+            plot_df <- plot_df[-which(plot_df$Class == "Total"),]
+            density_plot <-
+                ggplot(plot_df,
+                       aes_string(x = "Value", y = "..count..", fill = "Class", weight = "Probability")) +
+                scale_fill_manual(values = get_palette(length(levels(plot_df$Class))-1)) +
+                geom_density(position = "fill")
+        } else{
+            plot_df$alpha <- alpha
+            plot_df$alpha[plot_df$Class == "Total"] <- 0
+            plot_df$Class <- ordered(plot_df$Class, levels = c(levels(plot_df$Class)[-1][order(as.numeric(levels(plot_df$Class)[-1]))], levels(plot_df$Class)[1]))
+            density_plot <-
+                ggplot(plot_df,
+                       aes_string(x = "Value",
+                                  fill = "Class",
+                                  colour = "Class",
+                                  weight = "Probability",
+                                  alpha = "alpha"#,
+                                  #size = "size"
+                       ))
+            class_colors <- c(get_palette(length(levels(plot_df$Class))-1), "#000000")
+            density_plot <- density_plot +
+                scale_colour_manual(values = class_colors)+
+                scale_fill_manual(values = class_colors) +
+                scale_alpha_continuous(range = c(0, alpha), guide = FALSE)+
+                scale_size_continuous(range = c(.5, 1), guide = FALSE)+
+                geom_density()+
+                scale_x_continuous(expand = c(0, 0))+
+                scale_y_continuous(expand = c(0, 0))
+
+        }
+    }
+    density_plot
+}
